@@ -19,11 +19,13 @@
 
 pub(crate) mod parser;
 
-use {
-  self::errors::*,
-  dmntk_common::{DmntkError, HRef, OptHRef},
-  std::convert::TryFrom,
-};
+#[cfg(test)]
+mod tests;
+
+use self::errors::*;
+use dmntk_common::{DmntkError, HRef, OptHRef, Result};
+use dmntk_feel::{FeelType, Name};
+use std::convert::TryFrom;
 
 /// [Element] represents an element from another metamodel.
 #[derive(Debug, Clone, PartialEq)]
@@ -70,8 +72,18 @@ pub struct ExtensionAttribute {
 
 /// [NamedElement] adds the required attribute `name` to [DmnElement].
 pub trait NamedElement: DmnElement {
-  /// The name of this element.
+  /// Returns the name of this element.
   fn name(&self) -> &str;
+  /// Returns the optional `FEEL` name for this element.
+  fn feel_name(&self) -> &Option<Name>;
+}
+
+/// [FeelTypedElement] adds the `FEEL` type attributes to element.
+pub trait FeelTypedElement {
+  /// Returns the optional `FEEL` type for this element.
+  fn feel_type(&self) -> &Option<FeelType>;
+  /// Sets `FEEL` type for this element.
+  fn set_feel_type(&mut self, feel_type: FeelType);
 }
 
 /// The abstract class [BusinessContextElement], and its concrete specializations
@@ -105,6 +117,8 @@ pub struct PerformanceIndicator {
   extension_attributes: Vec<ExtensionAttribute>,
   /// Name of this [PerformanceIndicator].
   name: String,
+  /// Optional `FEEL` name of this [ItemDefinition].
+  feel_name: Option<Name>,
   /// The URI of this [PerformanceIndicator].
   uri: Option<String>,
   /// Collection of [Decision] that impact this [PerformanceIndicator].
@@ -140,6 +154,10 @@ impl NamedElement for PerformanceIndicator {
   fn name(&self) -> &str {
     &self.name
   }
+  /// Returns a reference to optional `FEEL` name of this element.
+  fn feel_name(&self) -> &Option<Name> {
+    &self.feel_name
+  }
 }
 
 impl BusinessContextElement for PerformanceIndicator {
@@ -169,6 +187,8 @@ pub struct OrganizationUnit {
   extension_attributes: Vec<ExtensionAttribute>,
   /// Name of this [OrganizationUnit].
   name: String,
+  /// Optional `FEEL` name of this [ItemDefinition].
+  feel_name: Option<Name>,
   /// The URI of this [OrganizationUnit].
   uri: Option<String>,
   /// Collection of [Decision] that are made by this [OrganizationUnit].
@@ -205,6 +225,10 @@ impl NamedElement for OrganizationUnit {
   fn name(&self) -> &str {
     &self.name
   }
+  /// Returns a reference to optional `FEEL` name of this element.
+  fn feel_name(&self) -> &Option<Name> {
+    &self.feel_name
+  }
 }
 
 impl BusinessContextElement for OrganizationUnit {
@@ -308,6 +332,8 @@ pub struct Definitions {
   extension_attributes: Vec<ExtensionAttribute>,
   /// Name of this [Definitions].
   name: String,
+  /// Optional `FEEL` name of this [ItemDefinition].
+  feel_name: Option<Name>,
   /// Identifies the namespace associated with this [Definitions]
   /// and follows the convention established by XML Schema.
   namespace: String,
@@ -337,6 +363,8 @@ pub struct Definitions {
   /// This attribute is not a part of specification,
   /// but is handy when importing definitions from other sources.
   source: String,
+  /// Optional DMNDI container.
+  dmndi: Option<Dmndi>,
 }
 
 impl Definitions {
@@ -364,6 +392,10 @@ impl Definitions {
   pub fn item_definitions(&self) -> &Vec<ItemDefinition> {
     &self.item_definitions
   }
+  /// Returns mutable reference to the container of instances of [ItemDefinition] contained in this [Definitions].
+  pub fn item_definitions_mut(&mut self) -> &mut Vec<ItemDefinition> {
+    &mut self.item_definitions
+  }
   /// Returns an optional reference to [ItemDefinition] with specified name
   /// or [None] when such [ItemDefinition] was not found in this [Definitions].
   pub fn item_definition_by_name(&self, name: &str) -> Option<&ItemDefinition> {
@@ -454,10 +486,26 @@ impl Definitions {
       None
     })
   }
+  ///
+  pub fn input_data(&self) -> Vec<&InputData> {
+    self
+      .drg_elements
+      .iter()
+      .filter_map(|v| if let DrgElement::InputData(input_data) = v { Some(input_data) } else { None })
+      .collect::<Vec<&InputData>>()
+  }
   /// Returns reference to the container of instances of [BusinessContextElement] contained in this [Definitions].
   pub fn business_context_elements(&self) -> &Vec<BusinessContextElementInstance> {
     &self.business_context_elements
   }
+  /// Returns reference to optional [Dmndi] container.
+  pub fn dmndi(&self) -> &Option<Dmndi> {
+    &self.dmndi
+  }
+  /// Returns reference to [DrgElements](DrgElement) container.
+  pub fn drg_elements_mut(&mut self) -> &mut Vec<DrgElement> {
+    &mut self.drg_elements
+  }
 }
 
 impl DmnElement for Definitions {
@@ -488,6 +536,10 @@ impl NamedElement for Definitions {
   fn name(&self) -> &str {
     &self.name
   }
+  /// Returns a reference to optional `FEEL` name of this element.
+  fn feel_name(&self) -> &Option<Name> {
+    &self.feel_name
+  }
 }
 
 #[derive(Debug, Clone, PartialEq)]
@@ -504,17 +556,23 @@ pub struct InformationItem {
   extension_attributes: Vec<ExtensionAttribute>,
   /// Name of this [InformationItem].
   name: String,
+  /// Optional `FEEL` name of this [ItemDefinition].
+  feel_name: Option<Name>,
   /// The `Expression` whose value is assigned to this [InformationItem].
   value_expression: Option<ExpressionInstance>,
   /// Optional qualified name of the type of this [InformationItem].
-  type_ref: String,
+  type_ref: Option<String>,
+  /// Optional `FEEL` type of this [InformationItem].
+  feel_type: Option<FeelType>,
 }
 
 impl InformationItem {
+  ///
   pub fn value_expression(&self) -> &Option<ExpressionInstance> {
     &self.value_expression
   }
-  pub fn type_ref(&self) -> &String {
+  ///
+  pub fn type_ref(&self) -> &Option<String> {
     &self.type_ref
   }
 }
@@ -547,6 +605,21 @@ impl NamedElement for InformationItem {
   fn name(&self) -> &str {
     &self.name
   }
+  /// Returns a reference to optional `FEEL` name of this element.
+  fn feel_name(&self) -> &Option<Name> {
+    &self.feel_name
+  }
+}
+
+impl FeelTypedElement for InformationItem {
+  /// Returns a reference to optional `FEEL` type of this element.
+  fn feel_type(&self) -> &Option<FeelType> {
+    &self.feel_type
+  }
+  /// Sets the `FEEL` type for this element.
+  fn set_feel_type(&mut self, feel_type: FeelType) {
+    self.feel_type = Some(feel_type);
+  }
 }
 
 /// [InputData] is used to model the inputs of a decision whose values
@@ -565,6 +638,8 @@ pub struct InputData {
   extension_attributes: Vec<ExtensionAttribute>,
   /// Name of this [InputData].
   name: String,
+  /// Optional `FEEL` name of this [ItemDefinition].
+  feel_name: Option<Name>,
   /// The instance of [InformationItem] that stores the result of this [InputData].
   variable: InformationItem,
 }
@@ -603,6 +678,10 @@ impl NamedElement for InputData {
   fn name(&self) -> &str {
     &self.name
   }
+  /// Returns a reference to optional `FEEL` name of this element.
+  fn feel_name(&self) -> &Option<Name> {
+    &self.feel_name
+  }
 }
 
 /// `Import` class is used when referencing external elements,
@@ -783,6 +862,8 @@ pub struct Decision {
   extension_attributes: Vec<ExtensionAttribute>,
   /// Name of the [Decision].
   name: String,
+  /// Optional `FEEL` name of this [ItemDefinition].
+  feel_name: Option<Name>,
   /// A natural language question that characterizes the [Decision],
   /// such that the output of the [Decision] is an answer to the question.
   question: Option<String>,
@@ -854,6 +935,10 @@ impl NamedElement for Decision {
   fn name(&self) -> &str {
     &self.name
   }
+  /// Returns a reference to optional `FEEL` name of this element.
+  fn feel_name(&self) -> &Option<Name> {
+    &self.feel_name
+  }
 }
 
 /// The class [InformationRequirement] is used to model an information requirement,
@@ -1062,6 +1147,8 @@ pub struct BusinessKnowledgeModel {
   extension_attributes: Vec<ExtensionAttribute>,
   /// Name of this [BusinessKnowledgeModel].
   name: String,
+  /// Optional `FEEL` name of this [ItemDefinition].
+  feel_name: Option<Name>,
   /// Variable that is bound to the function defined by the [FunctionDefinition] for this [BusinessKnowledgeModel].
   variable: InformationItem,
   /// The function that encapsulates the logic encapsulated by this [BusinessKnowledgeModel].
@@ -1115,6 +1202,10 @@ impl NamedElement for BusinessKnowledgeModel {
   fn name(&self) -> &str {
     &self.name
   }
+  /// Returns a reference to optional `FEEL` name of this element.
+  fn feel_name(&self) -> &Option<Name> {
+    &self.feel_name
+  }
 }
 
 impl Invocable for BusinessKnowledgeModel {
@@ -1140,6 +1231,8 @@ pub struct DecisionService {
   extension_attributes: Vec<ExtensionAttribute>,
   /// Name of this [DecisionService].
   name: String,
+  /// Optional `FEEL` name of this [ItemDefinition].
+  feel_name: Option<Name>,
   /// Variable for this [DecisionService].
   variable: InformationItem,
   /// Collection of references to the instances of [Decision] required to be output by this [DecisionService].
@@ -1199,6 +1292,10 @@ impl NamedElement for DecisionService {
   fn name(&self) -> &str {
     &self.name
   }
+  /// Returns a reference to optional `FEEL` name of this element.
+  fn feel_name(&self) -> &Option<Name> {
+    &self.feel_name
+  }
 }
 
 impl Invocable for DecisionService {
@@ -1208,6 +1305,17 @@ impl Invocable for DecisionService {
   }
 }
 
+/// Item definition types.
+#[derive(Debug, Clone)]
+pub enum ItemDefinitionType {
+  SimpleType(FeelType),
+  ReferencedType(String),
+  ComponentType,
+  CollectionOfSimpleType(FeelType),
+  CollectionOfReferencedType(String),
+  CollectionOfComponentType,
+}
+
 /// [ItemDefinition] is used to model the inputs of a decision,
 /// whose values are defined outside of the decision model.
 #[derive(Debug, Clone)]
@@ -1224,23 +1332,30 @@ pub struct ItemDefinition {
   extension_attributes: Vec<ExtensionAttribute>,
   /// Name of this [ItemDefinition].
   name: String,
+  /// Optional `FEEL` name of this [ItemDefinition].
+  feel_name: Option<Name>,
   /// Optional base type of this [ItemDefinition] identified by namespace-prefixed name.
   type_ref: Option<String>,
   /// This attribute identifies the type language used to specify the base
   /// type of this [ItemDefinition]. This value overrides the type
-  /// language specified in the [Definitions] element. The `Default` is `FEEL`.
-  /// The language `SHALL` be specified in a URI format.
+  /// language specified in the [Definitions] element. The default is `FEEL`.
+  /// The language `SHALL` be specified in an URI format.
   type_language: Option<String>,
+  /// This attribute contains `FEEL` built-in type only when the `type_language` attribute
+  /// is `FEEL` and the `type_ref` attribute defines one of the built-in `FEEL` types.
+  feel_type: Option<FeelType>,
   /// Possible values or ranges of values in the base type that are allowed in this [ItemDefinition].
   allowed_values: Option<UnaryTests>,
   /// Defines zero or more nested [ItemDefinitions](ItemDefinition) that compose this [ItemDefinition].
   item_components: Vec<ItemDefinition>,
   /// Setting this flag to true indicates that the actual values defined by
   /// this [ItemDefinition] are collections of allowed values.
-  /// The default is [false].
+  /// The default value is [false].
   is_collection: bool,
   /// Describes an optional [FunctionItem] that compose this [ItemDefinition].
   function_item: Option<FunctionItem>,
+  /// Optional item definition type.
+  item_definition_type: Option<ItemDefinitionType>,
 }
 
 impl ItemDefinition {
@@ -1257,10 +1372,30 @@ impl ItemDefinition {
   pub fn item_components(&self) -> &Vec<ItemDefinition> {
     &self.item_components
   }
+  /// Returns mutable reference to nested [ItemDefinitions](ItemDefinition) that compose this [ItemDefinition].
+  pub fn item_components_mut(&mut self) -> &mut Vec<ItemDefinition> {
+    &mut self.item_components
+  }
   /// Returns flag indicating if the actual values are collections of allowed values.
   pub fn is_collection(&self) -> bool {
     self.is_collection
   }
+  /// Returns a reference to optional `FEEL` type.
+  pub fn feel_type(&self) -> &Option<FeelType> {
+    &self.feel_type
+  }
+  /// Sets the `FEEL` type for this element.
+  pub fn set_feel_type(&mut self, feel_type: FeelType) {
+    self.feel_type = Some(feel_type);
+  }
+  /// Returns a reference to optional item definition type.
+  pub fn item_definition_type(&self) -> &Option<ItemDefinitionType> {
+    &self.item_definition_type
+  }
+  /// Sets the item definition type for this element.
+  pub fn set_item_definition_type(&mut self, item_definition_type: ItemDefinitionType) {
+    self.item_definition_type = Some(item_definition_type);
+  }
 }
 
 impl DmnElement for ItemDefinition {
@@ -1291,6 +1426,10 @@ impl NamedElement for ItemDefinition {
   fn name(&self) -> &str {
     &self.name
   }
+  /// Returns a reference to optional `FEEL` name of this element.
+  fn feel_name(&self) -> &Option<Name> {
+    &self.feel_name
+  }
 }
 
 impl Expression for ItemDefinition {
@@ -1789,10 +1928,247 @@ pub struct AnnotationEntry {
   pub text: String,
 }
 
+/// [Dmndi] is a container for the shared [DmnStyle](DmnStyle)s
+/// and all [DmnDiagram](DmnDiagram)s defined in [Definitions].
+#[derive(Debug, Clone)]
+pub struct Dmndi {
+  /// A list of shared [DmnStyle] that can be referenced
+  /// by all [DmnDiagram] and [DmnDiagramElement].
+  pub styles: Vec<DmnStyle>,
+  /// A list of [DmnDiagram].
+  pub diagrams: Vec<DmnDiagram>,
+}
+
+/// Defines possible elements of [DmnDiagramElement].
+#[derive(Debug, Clone)]
+pub enum DmnDiagramElement {
+  DmnShape(DmnShape),
+  DmnEdge(DmnEdge),
+}
+
+/// [DmnDiagram] is the container of [DmnDiagramElement] ([DmnShape] (s) and [DmnEdge] (s)).
+/// [DmnDiagram] cannot include other [DmnDiagrams](DmnDiagram).
+#[derive(Debug, Clone)]
+pub struct DmnDiagram {
+  /// [DmnDiagram] id.
+  pub id: Option<String>,
+  /// The name of the diagram. Default is empty [String].
+  pub name: String,
+  /// The documentation of the diagram. Default is empty [String].
+  pub documentation: String,
+  /// The resolution of the diagram expressed in user units per inch. Default is 300.
+  pub resolution: f64,
+  /// A list of [DmnDiagramElement] ([DmnShape] and [DmnEdge]) that are depicted in this diagram.
+  pub diagram_elements: Vec<DmnDiagramElement>,
+  /// A reference to a [DmnStyle] defined in the [Dmndi] that serves as the default styling
+  /// of the [DmnDiagramElement] in this [DmnDiagram].
+  pub shared_style: Option<String>,
+  /// A [DmnStyle] that defines the default styling for this diagram.
+  /// Properties defined in that style override the ones in the 'sharedStyle'.
+  pub local_style: Option<DmnStyle>,
+  /// The size of this diagram. If not specified, the [DmnDiagram] is unbounded.
+  pub size: Option<DcDimension>,
+}
+
+/// [DmnShape] represents a [Decision], a [BusinessKnowledgeModel], an [InputData] element,
+/// a [KnowledgeSource], a [DecisionService] or a [TextAnnotation] that is depicted on the diagram.
+#[derive(Debug, Clone)]
+pub struct DmnShape {
+  /// [DmnShape] id.
+  pub id: Option<String>,
+  /// The [DcBounds] of the shape relative to the origin of its parent [DmnDiagram]. The [DcBounds] MUST be specified.
+  pub bounds: DcBounds,
+  /// A  reference  to  a  [Decision], a [BusinessKnowledgeModel], an [InputData] element,
+  /// a [KnowledgeSource], a [DecisionService] or a [TextAnnotation] MUST be specified.
+  pub dmn_element_ref: Option<String>,
+  /// If the [DmnShape] depicts an [InputData] element then this attribute is used to determine
+  /// if the [InputData] is listed on the [Decision] element (true) or drawn as separate notational elements in the DRD (false).
+  pub is_listed_input_data: bool,
+  /// If the [DmnShape] depicts a [DecisionService], this attribute references a [DmnDecisionServiceDividerLine] that defines,
+  /// where the [DmnShape] is divided into two parts by a straight solid line.
+  /// This can be the case when a [DmnShape] depicts a [DecisionService],
+  /// where the set of output decisions is smaller than the set of encapsulated decisions.
+  /// The start and end waypoints of the `decisionServiceDividerLine` **MUST** be on the border of the [DmnShape].
+  pub decision_service_divider_line: Option<DmnDecisionServiceDividerLine>,
+  /// If the [DmnShape] depicts a [DecisionService], this attribute indicates
+  /// if it should be depicted expanded (`false`) or collapsed (`true`).
+  /// Default is `false`.
+  pub is_collapsed: bool,
+  /// A reference to a [DmnStyle] defined in the [Dmndi].
+  pub shared_style: Option<String>,
+  /// A [DmnStyle] that defines the styling for this element.
+  pub local_style: Option<DmnStyle>,
+  /// An optional label when this [DmnElement] has a visible text label.
+  pub label: Option<DmnLabel>,
+}
+
+/// Struct defines line inside [DecisionService].
+#[derive(Debug, Clone)]
+pub struct DmnDecisionServiceDividerLine {
+  pub id: Option<String>,
+  /// A list of points relative to the origin of its parent [DmnDiagram] that specifies
+  /// the connected line segments of the edge. At least two (2) waypoint`s MUST be specified.
+  /// Waypoint must be on the border of the [DmnShape].
+  pub way_points: Vec<DcPoint>,
+  /// A reference to a [DmnStyle] defined in the [Dmndi].
+  pub shared_style: Option<String>,
+  /// A [DmnStyle] that defines the styling for this element.
+  pub local_style: Option<DmnStyle>,
+}
+
+#[derive(Debug, Clone)]
+pub struct DmnEdge {
+  pub id: Option<String>,
+  /// A list of points relative to the origin of its parent [DmnDiagram] that specifies
+  /// the connected line segments of the edge. At least two (2) waypoints MUST be specified.
+  pub way_points: Vec<DcPoint>,
+  /// A reference to a [InformationRequirement], [KnowledgeRequirement],
+  /// [AuthorityRequirement] or an [Association] MUST be specified.
+  pub dmn_element_ref: Option<String>,
+  /// The actual [DmnDiagramElement] this [DmnEdge] is connecting from.
+  /// MUST be specified when the [DmnEdge] has a source.
+  pub source_element: Option<String>,
+  /// The actual [DmnDiagramElement] this [DmnEdge] is connecting to.
+  /// MUST be specified when the [DmnEdge] has a target.
+  pub target_element: Option<String>,
+  /// A reference to a [DmnStyle] defined in the [Dmndi].
+  pub shared_style: Option<String>,
+  /// A [DmnStyle] that defines the styling for this element.
+  pub local_style: Option<DmnStyle>,
+  /// An optional label when this [DmnElement] has a visible text label.
+  pub label: Option<DmnLabel>,
+}
+
+//FIXME verify this struct
+/// tdb
+#[derive(Debug, Clone)]
+pub struct Association {}
+
+//FIXME verify this struct
+/// tdb
+#[derive(Debug, Clone)]
+pub struct TextAnnotation {}
+
+/// [DmnStyle] is used to keep some non-normative visual attributes such as colors and font.
+#[derive(Debug, Clone)]
+pub struct DmnStyle {
+  /// A unique id for this style so it can be referenced.
+  /// Only styles defined in the [Dmndi] can be referenced by [DmnDiagramElement] and [DmnDiagram].
+  pub id: Option<String>,
+  /// The color use to fill the shape. Does not apply to [DmnEdge].
+  /// Default is `white`.
+  pub fill_color: Option<DcColor>,
+  /// The color use to draw the shape borders.
+  /// Default is `black`.
+  pub stroke_color: Option<DcColor>,
+  /// The color use to draw the label.
+  /// Default is `black`.
+  pub font_color: Option<DcColor>,
+  /// A comma-separated list of Font Name that can be used to display the text.
+  /// Default is `Arial`.
+  pub font_family: String,
+  /// The size in points of the font to use to display the text.
+  /// Default is `8`.
+  pub font_size: f64,
+  /// If the text should be displayed in Italic.
+  /// Default is `false`.
+  pub font_italic: bool,
+  /// If the text should be displayed in Bold.
+  /// Default is `false`.
+  pub font_bold: bool,
+  /// If the text should be underlined.
+  /// Default is `false`.
+  pub font_underline: bool,
+  /// If the text should be stroke through.
+  /// Default is `false`.
+  pub font_strike_through: bool,
+  /// How text should be positioned horizontally within the Label bounds.
+  /// Default depends of the [DmnDiagramElement] the label is attached to (see 13.5).
+  pub label_horizontal_alignment: Option<DcAlignmentKind>,
+  /// How  the  text  should  be  positioned  vertically  inside  the  Label  bounds.
+  /// Default depends of the [DmnDiagramElement] the label is attached to (see 13.5).
+  /// Start means `top` and end means `bottom`.
+  pub label_vertical_alignment: Option<DcAlignmentKind>,
+}
+
+/// Struct represents the depiction of some textual information about a DMN element.
+#[derive(Debug, Clone)]
+pub struct DmnLabel {
+  /// The bounds of the [DmnLabel]. When not specified, the label is positioned
+  /// at its default position as determined in clause 13.5.
+  pub bounds: Option<DcBounds>,
+  /// An optional pretty printed text that MUST be displayed
+  /// instead of the [DmnElement's](DmnElement) name if it is present.
+  pub text: Option<String>,
+  /// A reference to a [DmnStyle] defined in the [Dmndi].
+  pub shared_style: Option<String>,
+}
+
+/// Defines RGB color.
+#[derive(Debug, Clone)]
+pub struct DcColor {
+  pub red: u8,
+  pub green: u8,
+  pub blue: u8,
+}
+
+/// Defines point.
+#[derive(Debug, Clone)]
+pub struct DcPoint {
+  pub x: f64,
+  pub y: f64,
+}
+
+/// Defines bounds.
+#[derive(Debug, Clone)]
+pub struct DcBounds {
+  pub x: f64,
+  pub y: f64,
+  pub width: f64,
+  pub height: f64,
+}
+
+/// Defines dimensions.
+#[derive(Debug, Clone)]
+pub struct DcDimension {
+  pub width: f64,
+  pub height: f64,
+}
+
+/// Defines element kind alignment.
+#[derive(Debug, Clone)]
+pub enum DcAlignmentKind {
+  Start,
+  End,
+  Center,
+}
+
+/// Defines known colors.
+#[derive(Debug, Clone)]
+pub enum DcKnownColor {
+  Maroon = 0x800000,
+  Red = 0xFF0000,
+  Orange = 0xFFA500,
+  Yellow = 0xFFFF00,
+  Olive = 0x808000,
+  Purple = 0x800080,
+  Fuchsia = 0xFF00FF,
+  White = 0xFFFFFF,
+  Lime = 0x00FF00,
+  Green = 0x008000,
+  Navy = 0x000080,
+  Blue = 0x0000FF,
+  Aqua = 0x00FFFF,
+  Teal = 0x008080,
+  Black = 0x000000,
+  Silver = 0xC0C0C0,
+  Gray = 0x808080,
+}
+
 mod errors {
   use dmntk_common::DmntkError;
 
-  /// Errors related the model.
+  /// Errors related to the model.
   #[derive(Debug, PartialEq)]
   pub enum ModelError {
     InvalidDecisionTableOrientation(String),
